import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { SaleReport } from './sale-report';

const url = '/api/sale-report';
const serviceName = 'SaleReportService';

@Injectable({
  providedIn: 'root',
})
export class SaleReportService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(startDate: string | null, finishDate: string | null): Observable<SaleReport> {
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<SaleReport>>(
      this.http
        .get<SaleReport>(url, options)
        .pipe(catchError(this.log.handleError(serviceName, 'get')))
    );
  }

  print(startDate: string | null, finishDate: string | null): Observable<boolean> {
    const printUrl = `${url}/print`;
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<boolean>>(
      this.http
        .get<boolean>(printUrl, options)
        .pipe(catchError(this.log.handleError(serviceName, 'print')))
    );
  }
}
